@@ -1,4 +1,4 @@
-;;; $Id: xmls.lisp 542 2010-02-24 19:25:39Z rpgoldman $
+;;; $Id: xmls.lisp 609 2011-02-23 21:11:46Z rpgoldman $
 ;;; xmls
 ;;; a simple xml parser for common lisp
 ;;; author: Miles Egan <miles@caddr.com>
@@ -109,13 +109,32 @@
             str))
       str))
 
+;; (defun write-escaped (string stream)
+;;   "Writes string to stream with all character entities escaped."
+;;   #-allegro (coerce string 'simple-base-string)
+;;   (when (eq stream t) (setf stream *standard-output*))
+;;   (loop for char across string
+;;         for esc = (svref *char-escapes* (char-code char))
+;;         do (write-sequence esc stream)))
+
+;;; Alternative definition, lifted from Edi Weitz's hunchentoot, per Norman
+;;; Werner's suggestion. [2010/12/09:rpg]
 (defun write-escaped (string stream)
-  "Writes string to stream with all character entities escaped."
-  #-allegro (coerce string 'simple-base-string)
-  (when (eq stream t) (setf stream *standard-output*))
-  (loop for char across string
-        for esc = (svref *char-escapes* (char-code char))
-        do (write-sequence esc stream)))
+  (write-string (escape-for-html string) stream))
+
+(defun escape-for-html (string)
+  "Escapes the characters #\\<, #\\>, #\\', #\\\", and #\\& for HTML output."
+  (with-output-to-string (out)
+    (with-input-from-string (in string)
+      (loop for char = (read-char in nil nil)
+            while char
+            do (case char
+                 ((#\<) (write-string "&lt;" out))
+                 ((#\>) (write-string "&gt;" out))
+                 ((#\") (write-string "&quot;" out))
+                 ((#\') (write-string "&#039;" out))
+                 ((#\&) (write-string "&amp;" out))
+                 (otherwise (write-char char out)))))))
 
 (defun make-extendable-string (&optional (size 10))
   "Creates an adjustable string with a fill pointer."
@@ -228,6 +247,10 @@ character translation."
   "Consumes one character from the input stream."
   `(read-char (state-stream s)))
 
+(defmacro puke (char)
+  "The opposite of EAT."
+  `(unread-char ,char (state-stream s)))
+
 (defmacro match (&rest matchers)
   "Attempts to match the next input character with one of the supplied matchers."
   `(let ((c (peek-stream (state-stream s))))
@@ -465,35 +488,107 @@ character translation."
      t)
    (make-element :type 'comment)))
 
+;;; the following is buggy, because it does not properly back up when it gets a
+;;; mismatch.  An example buggy string is: "<name><![CDATA[x]]]></name>"
+;;; [2011/02/21:rpg]
+;; (defrule comment-or-cdata ()
+;;   (and
+;;    (peek #\!)
+;;    (must (or (comment s)
+;;              (and
+;;               (match-seq #\[ #\C #\D #\A #\T #\A #\[)
+;;               (loop with data = (make-extendable-string 50)
+;;                     with state = 0
+;;                     do (case state
+;;                          (0 (cond ((match #\])
+;;                                    (norvig:dbg :cdata "Match #\], go to state 1.")
+;;                                    (incf state))
+;;                                   (t
+;;                                    (push-string (eat) data))))
+;;                          (1 (cond ((match #\])
+;;                                    (norvig:dbg :cdata "Match second #\], go to state 2.")
+;;                                    (incf state))
+;;                                   (t
+;;                                    (norvig:dbg :cdata "Fail to match second #\], go to state 0.")
+;;                                    (setf state 0)
+;;                                    ;; dump the first close-bracket
+;;                                    (push-string #\] data)
+;;                                    ;; just go back to the matching process [2011/02/21:rpg]
+;;                                    ;; (push-string (eat) data)
+;;                                    )))
+;;                          (2 (cond ((match #\>)
+;;                                    (norvig:dbg :cdata "Finish closing of CDATA.")
+;;                                    (incf state))
+;;                                   (t
+;;                                    (norvig:dbg :cdata "Fail to find >, return to state 0.")
+;;                                    (setf state 0)
+;;                                    ;; the FIRST close-bracket doesn't start a match
+;;                                    (push-string #\] data)
+;;                                    ;; start reading again from the second close-bracket, which might
+;;                                    ;; start a match... [2011/02/21:rpg]
+;;                                    (puke #\])
+;;                                    ;; (push-string (eat) data)
+;;                                    ))))
+;;                     until (eql state 3)
+;;                     finally (return (make-element
+;;                                      :type 'cdata
+;;                                      :val (coerce data 'simple-string)))))))))
+
+;;; I was unable to figure out how to rejigger this backtracking lexer because
+;;; of the possible need to do multiple step backup.  Instead, for the CDATA
+;;; matching of ]]> I by hand generated an NFA, and then determinized it (also
+;;; by hand).  Then I did a simpler thing of just pushing ALL the data onto the
+;;; data string, and truncating it when done.
 (defrule comment-or-cdata ()
-  (and
-   (peek #\!)
-   (must (or (comment s)
-             (and
-              (match-seq #\[ #\C #\D #\A #\T #\A #\[)
-              (loop with data = (make-extendable-string 50)
-                    with state = 0
-                    do (case state
-                         (0 (if (match #\])
-                                (incf state)
-                                (push-string (eat) data)))
-                         (1 (if (match #\])
-                                (incf state)
-                                (progn
-                                  (setf state 0)
-                                  (push-string #\] data)
-                                  (push-string (eat) data))))
-                         (2 (if (match #\>)
-                                (incf state)
-                                (progn
-                                  (setf state 0)
-                                  (push-string #\] data)
-                                  (push-string #\] data)
-                                  (push-string (eat) data)))))
-                    until (eq state 3)
-                    finally (return (make-element
-                                     :type 'cdata
-                                     :val (coerce data 'simple-string)))))))))
+  (macrolet ((dbg (&rest args)
+               #+xmls-debug
+               `(norvig:dbg ,@args)
+               #-xmls-debug
+               nil))
+    (and
+     (peek #\!)
+     (must (or (comment s)
+               (and
+                (match-seq #\[ #\C #\D #\A #\T #\A #\[)
+                (loop with data = (make-extendable-string 50)
+                      with state = 0
+                      for char = (eat)
+                      do (push-string char data)
+                      do (case state
+                           (0
+                            (case char
+                              (#\]
+                               (dbg :cdata "State 0 Match #\], go to state {0,1} = 4.")
+                               (setf state 4))
+                              (otherwise
+                               (dbg :cdata "State 0 Non-], go to (remain in) state 0."))))
+                           (4 ; {0, 1}
+                            (case char
+                              (#\]
+                               (dbg :cdata "State 4 {0, 1}, match ], go to state {0,1,2} = 5")
+                               (setf state 5))
+                              (otherwise
+                               (dbg :cdata "State 4 {0, 1}, Non-], go to state 0.")
+                               (setf state 0))))
+                           (5 ; {0, 1, 2}
+                            (case char
+                              (#\]
+                               (dbg :cdata "State 5 {0, 1, 2}, match ], stay in state 5."))
+                              (#\>
+                               (dbg :cdata "State 5 {0, 1, 2}, match >, finish match and go to state 3.")
+                               (setf state 3))
+                              (otherwise
+                               (dbg :cdata "State 5 {0, 1, 2}, find neither ] nor >; go to state 0.")
+                               (setf state 0))))
+                           )
+                      until (eql state 3)
+                      finally (return (make-element
+                                       :type 'cdata
+                                       :val (coerce
+                                             ;; rip the ]]> off the end of the data and return it...
+                                             (subseq data 0 (- (fill-pointer data) 3))
+                                             'simple-string))))))))))
+
 
 (declaim (ftype function element))     ; forward decl for content rule
 (defrule content ()
@@ -624,14 +719,19 @@ character translation."
 ;;(trace end-tag comment comment-or-doctype content name xmldecl misc)
 ;;(trace processing-instruction processing-instruction-or-xmldecl element start-tag ws element-val)
 
-#+(or sbcl cmu allegro)
+#+(or sbcl cmu allegro abcl ccl)
 (defun test ()
   ;;(sb-profile:profile "XMLS")
   #+cmu(extensions:gc-off) ;; too noisy
-  (dolist (test (cdr
-                 #+sbcl sb-ext:*posix-argv*
-                 #+cmu (subseq extensions:*command-line-strings* 4)
-                 #+allegro (sys:command-line-arguments)))
+  (dolist (test
+            #-ccl
+           (cdr
+            #+sbcl sb-ext:*posix-argv*
+            #+abcl extensions:*command-line-argument-list*
+            #+cmu (subseq extensions:*command-line-strings* 4)
+            #+allegro (sys:command-line-arguments))
+           #+ccl
+           ccl:*unprocessed-command-line-arguments*)
     (handler-bind ((error #'(lambda (c)
                               (format t "FAILED with error:~%~S~%" c)
                               (throw 'test-failure nil))))
@@ -645,7 +745,18 @@ character translation."
                 (if (parse (open test))
                     (format t "ok~%")
                     (format t "FAILED!~%"))))))))
+      (handler-bind ((error #'(lambda (c)
+                              (format t "FAILED with error:~%~S~%" c)
+                              (throw 'test-failure nil))))
+        (catch 'test-failure
+          (format t "~40A" "Escaped writing...")
+          (force-output)
+          (with-output-to-string (str)
+          (write-escaped "ÄΩ" str))
+          (format t "ok~%")))
   ;;(sb-profile:report)
-  #+sbcl(sb-ext:quit)
+  #+abcl (extensions:quit)
+  #+ccl (ccl:quit)
+  #+sbcl (sb-ext:quit)
   #+cmu(extensions:quit)
   #+allegro(excl:exit))
